function rhs = cstrode(x, u, p, pars)
    % Nonlinear ODE model for reactor.
    c = x(1);
    T = x(2);
    h = x(3) + eps(); % Avoids division by zero.

    Tc = u(1);
    F = u(2);

    F0 = p(1);

    k = pars.k0*exp(-pars.E/T);
    rate = k*c;

    dcdt = F0*(pars.c0 - c)/(pars.A*h) - rate;
    dTdt = F0*(pars.T0 - T)/(pars.A*h) ...
           - pars.DeltaH/pars.rhoCp*rate ...
           + 2*pars.U/(pars.r*pars.rhoCp)*(Tc - T);
    dhdt = (F0 - F)/pars.A;

    rhs = [dcdt; dTdt; dhdt];