function l = vertices2lines(p, xlims, ylims)
    % l = vertices2lines(p, xlims, ylims)
    %
    % Returns 2-column array of line segments corresponding to edges of the
    % polygon with vertices in columns of p (which must be in the proper order).
    % Each line segment has 101 interpolation points (to make gnuplot happy),
    % and there is a row of NaNs inserted between each line.
    dp = diff(p, 1, 2);
    N = size(p, 2) - 1;
    blocks = cell(N, 1);
    xlims = linspace(xlims(1), xlims(2), 101)';
    ylims = linspace(ylims(1), ylims(2), 101)';
    for i = 1:N
        slope = dp(2,i)/dp(1,i);
        if isinf(slope) || isnan(slope)
            x = p(1,i)*[1; 1];
            y = ylims;
        else
            x = xlims;
            y = slope*(xlims - p(1,i)) + p(2,i);
        end
        blocks{i} = [[x, y]; NaN(1, 2)];
    end
    l = vertcat(blocks{:});