% Set computations for Ex. 2.6, 2.7, and 2.8.

% Define model, cost function, and bounds.
A = [2, 1; 0, 2];
B = [1, 0; 0, 1];
N = 3;

alpha = 1e-5;
Q = alpha*eye(2);
R = eye(2);

% Bounds.
xlb = [-5; -inf()];
xub = [5; inf()];
ulb = [-1; -1];
uub = [1; 1];

% Find LQR.

[K, P] = dlqr(A, B, Q, R);
K = -K; % Sign convention.

% Define helper functions.

% Run the computation for a few different cases.
terminals = {'termeq', 'lqr'};
Xn = struct();
V = struct();
Z = struct();
for i = 1:length(terminals)
    t = terminals{i};
    [Xn.(t), V.(t), Z.(t)] = findXn(A, B, K, N, xlb, xub, ulb, uub, t);
    title(t);
end

% Simulate MPC from the initial starting point.
model = struct('A', A, 'B', B, 'N', N);
constraint = Z.lqr;
penalty = struct('Q', Q, 'R', R, 'P', P);
terminal = Xn.lqr{1}; % LQR terminal set.

Nsim = 25;
xsim = zeros(2, Nsim + 1);
xsim(:,1) = [0.5; 0.5]; % Initial condition.
usim = zeros(2, Nsim);
mpcmats = []; % Calculated first time and then reused.
for t = 1:Nsim
    model.x0 = xsim(:,t);
    [xk, uk, ~, status, mpcmats] = linearmpc(model, constraint, penalty, ...
                                             terminal, mpcmats);
    usim(:,t) = uk(:,1);
    xsim(:,t + 1) = xk(:,2);
end

% Now check and see where the finite-horizon control law is the same as the
% infinite-horizon control law.
[x1, x2] = meshgrid(linspace(-1.86, 1.86, 50), linspace(-0.97, 0.97, 50));
x0 = [x1(:), x2(:)]';
terminal = Xn.lqr{1};
X3 = Xn.lqr{end}; % Feasible set.
okaypts = all(bsxfun(@le, X3.A*x0, X3.b));
x0 = x0(:,okaypts); % Get rid of guys that are not feasible to save time.

Npts = size(x0, 2);
ininterior = all(bsxfun(@le, terminal.A*x0, terminal.b)); % See if start in Xf.
mpcmats = [];
for i = 1:Npts
    if ~ininterior(i)
        % Need to check with MPC problem.
        model.x0 = x0(:,i);
        [xk, uk, ~, status, mpcmats] = linearmpc(model, constraint, penalty, ...
                                                 terminal, mpcmats);
        if status.optimal
            % Check whether the ending state is in the interior (with tolerance).
            ininterior(i) = all(terminal.A*xk(:,end) < terminal.b - 1e-3);
        end
    end
end
x0interior = x0(:,ininterior);

% Finally, save data files.
lims = [-10; 10];

pa = V.lqr{1}'; % X_f for LQR (vertices)
palines = vertices2lines(pa', lims, lims); % X_f for LQR (facets)

pb = V.lqr{end}'; % X_n for LQR (vertices)
pblines = vertices2lines(pb', lims, lims); % 'X_n for LQR (facets)

pc = V.termeq{end}'; % X_n for terminal constraint (vertices)

xtable = [0:Nsim; xsim]'; % x timeseries using X_f for LQR

utable = [0:(Nsim - 1); usim]'; % u timeseries using X_f for LQR

pe = x0interior'; % x coordinates where finite-horizon = inf-horizon

save('Xinf.dat', 'pa', 'palines', 'pb', 'pblines', 'pc', ...
     'xtable', 'utable', 'pe');
