function V = cost(u, th, firstsign)
    % Computes optimal cost with inf if infeasible.
    narginchk(3, 3);
    x = cos(pi()*th);
    y = sin(pi()*th);

    % Calculate b.
    bnum = 3*u.^3 - 3*x.*u.^2 - 3*x.^2.*u - x.^3 + 4*y;
    bden = 12*(u + x);
    b = bnum./bden;

    % Simulate the system.
    useq = {u, -x/2 - u/2 + firstsign*sqrt(b), -x/2 - u/2 - firstsign*sqrt(b)};
    V = 0;
    for i = 1:4
        V = V + x.^2 + y.^2;
        if i == 4
            break
        end
        V = V + useq{i}.^2;
        x = x + useq{i};
        y = y + useq{i}.^3;
    end
    V = real(V);
    V(b < 0) = inf(); % These points are infeasible.