function [pieces, jumps] = getdiscont(x, y, N)
    % Finds discontinuities in y and adds in NaNs to make a gap.
    dy = abs(diff(y));
    idiscont = find(dy > mean(dy) + N*std(dy));

    % Interpolate x and y, screening out any large jumps.
    expand = @(z) [z; z(1:end-1) + 0.5*diff(z), NaN()];
    x = expand(x);
    xdc1 = x(1,idiscont);
    xdc2 = x(1,idiscont + 1);
    x(2,idiscont) = NaN();

    y = expand(y);
    ydc1 = y(1,idiscont);
    ydc2 = y(1,idiscont + 1);
    y(2,idiscont) = NaN();

    % Assemble discontinuities for plotting.
    xdc = [xdc1; xdc2; NaN(size(xdc2))];
    ydc = [ydc1; ydc2; NaN(size(ydc2))];

    % Package everybody up.
    pieces = [x(:), y(:)];
    jumps = [xdc(:), ydc(:)];