function Xn = calcXn(A, B, f, N, Xf, U, x0)
    % Calculate X_n sets for the system
    %
    %     x^+ = Ax + Bu + f
    %
    % Xf should be an extreme point representation of Xf, and U should be a
    % cell array of extreme-point polytopes for U.
    narginchk(6, 6);
    Ainv = inv(A);
    BU = cellfun(@(u) B*u + f, U, 'UniformOutput', false());
    Xn = cell(N + 1, 1);
    Xn{1} = {Xf};
    fprintf('*** Calculating Xk (max k = %d) ***\n', N);
    for n = 2:(N + 1)
        Xn{n} = cell(length(Xn{n - 1}), length(BU));
        for i = 1:length(Xn{n - 1})
            xn = Xn{n - 1}{i};
            for j = 1:length(BU)
                bu = BU{j};
                Xn{n}{i, j} = Ainv*minkowski(xn, -bu);
            end
        end
        Xn{n} = Xn{n}(:); % Flatten to a single list.
        fprintf('   k = %d: %d regions\n', n - 1, length(Xn{n}));
    end