function Xf = ellipseterm(A, P, xss, rho, n)
    % Xf = ellipseterm(A, P, xss, rho, n)
    %
    % Returns a polyhedral tangent approximation of an ellipsoidal terminal set.
    % Also makes a plot of the evolution of Xf under A.
    %
    % Xf is a 2 by n matrix of extreme points. A and b are the polyhedral
    % representation of Xf. P is the terminal penalty weight.
    narginchk(5, 5);

    [v, l] = eig(P/rho);

    a = 1/sqrt(l(1,1));
    b = 1/sqrt(l(2,2));
    calc = @(t) v(:,1)*a*cos(t) + v(:,2)*b*sin(t);

    T = linspace(0, 2*pi(), n + 1);
    t = linspace(0, 2*pi(), 10*n + 1);

    E = calc(T);
    AE = A*E;

    e = calc(t);
    Ae = A*e;

    figure();
    hold('on');
    plot(E(1,:) + xss(1), E(2,:) + xss(2), '-ob', 'DisplayName', 'X_f');
    plot(e(1,:) + xss(1), e(2,:) + xss(2), '--b');
    plot(AE(1,:) + xss(1), AE(2,:) + xss(2), '-or', ...
         'DisplayName', 'f(X_f, \kappa_f(X_f))');
    plot(Ae(1,:) + xss(1), Ae(2,:) + xss(2), '--r');
    xlabel('T_1');
    ylabel('T_2', 'rotation', 0);
    title(sprintf('Terminal Region X_f = {x | x''Px <= %g}', rho));
    legend('location', 'NorthEast');

    Xf = bsxfun(@plus, xss, E(:,1:(end - 1)));