% Calculation of X_n sets for a linear system with discrete actuators.
% Note that for computational purposes, we use a polyhedral approximation of
% the true elliptical X_n.



sys = getcooler();
Qmax = sys.Qmax;
Qmin = sys.Qmin;
ns = sys.ns;
A = sys.A;
B = sys.B(:,1); % Second input doesn't affect dynamics.
F = sys.F;
P = sys.Xf.P;
xss = sys.xss;
rho = sys.Xf.rho;

Xf = ellipseterm(A, P, xss, rho, 32);




% Calculate feasible sets.
Ucontinuous = {[0, max(ns)*Qmax]};
Udiscrete = arrayfun(@(n) n*[Qmin, Qmax], ns, 'UniformOutput', false());

N = 7;
Xc = calcXn(A, B, F, N, Xf, Ucontinuous);
Xd = calcXn(A, B, F, N, Xf, Udiscrete);

% Make a plot.

fprintf('Plotting. May take some time.\n');
figure();
lims = [sys.T1ss + 25*[-1, 1], sys.T2ss + 15*[-1, 1]];

subplot(1, 2, 1);
plotXn(Xc, lims);
title('Q_{min} = 0, Q_{max} = 10');

subplot(1, 2, 2);
plotXn(Xd, lims);
title('Q_{min} = 9, Q_{max} = 10');

