function data = run_standardvstube(standard_pars, tube_pars, Nsamples)
% Runs standard and tube MPC for problem and make resulting plot of
% trajectories.

standardMPCdata = struct();
tubeMPCdata = struct();

for i = 1:Nsamples
    fprintf('*** Sample %d of %d.\n', i, Nsamples);

    % Disturbance for this realization.
    standard_pars.A = standard_pars.A_rand(i);
    standard_pars.w = standard_pars.w_rand(i);
    tube_pars.A = tube_pars.A_rand(i);
    tube_pars.w = tube_pars.w_rand(i);

    % Run standard MPC.
    result = standardmpc(standard_pars);
    standardMPCdata.x1(:,i) = (result.x(1,:))';
    standardMPCdata.x2(:,i) = (result.x(2,:))';
    standardMPCdata.x1plant(:,i) = (result.xplant(1,:))';
    standardMPCdata.x2plant(:,i) = (result.xplant(2,:))';
    standardMPCdata.u(:,i) = result.u';
    standardMPCdata.pars = result.pars;

    % Run tube-based MPC.
    result = tubempc(tube_pars);
    tubeMPCdata.x1(:,i) = (result.x(1,:))';
    tubeMPCdata.x2(:,i) = (result.x(2,:))';
    tubeMPCdata.x1plant(:,i) = (result.xplant(1,:))';
    tubeMPCdata.x2plant(:,i) = (result.xplant(2,:))';
    tubeMPCdata.u(:,i) = result.u';
    tubeMPCdata.pars = result.pars;
end

% Make plot.
standardMPCdata.t = result.t;
standardMPCdata.tplant = result.tplant;
tubeMPCdata.t = result.t;
tubeMPCdata.tplant = result.tplant;
figure();

subplot(3,2,1)
plot(standardMPCdata.tplant, standardMPCdata.x1plant, 'k-')
ylabel('concentration')
title('standard MPC')
subplot(3,2,3)
plot(standardMPCdata.tplant, standardMPCdata.x2plant, 'k-')
ylabel('temperature')
subplot(3,2,5)
plot(standardMPCdata.t(1:end-1), standardMPCdata.u, 'k-')
ylabel('coolant flow')
xlabel('time')
subplot(3,2,2)
plot(standardMPCdata.tplant, tubeMPCdata.x1plant, 'k-')
ylabel('concentration')
title('tube-based MPC')
subplot(3,2,4)
plot(standardMPCdata.tplant, tubeMPCdata.x2plant, 'k-')
ylabel('temperature')
subplot(3,2,6)
plot(standardMPCdata.t(1:end-1), tubeMPCdata.u, 'k-')
ylabel('coolant flow')
xlabel('time')

% Output Results.
data = struct();
data.standard = standardMPCdata;
data.tube = tubeMPCdata;

end%function
