function data = doekf(Nsim, clipping)
    % data = doekf(Nsim, clipping)
    %
    % Simulate EKF with or without clipping.
    pars = batchreactor(Nsim);
    Nx = pars.Nx;
    Ny = pars.Ny;
    Nw = pars.Nw;

    % Get linearization functions.
    f = pars.f;
    Afunc = f.factory('Afunc', f.name_in(), 'jac:f:x');
    Gfunc = f.factory('Gfunc', f.name_in(), 'jac:f:w');
    h = pars.h;
    Cfunc = h.factory('Cfunc', h.name_in(), 'jac:h:x');

    % Choose simulation parameters.
    Nsim = pars.Nsim;
    t = pars.t;
    x = pars.xsim;
    y = pars.ysim;
    Qw = pars.Q;
    Rv = pars.R;

    % Try ekf.
    xhat = NaN(Nx, Nsim + 1);
    yhat = NaN(Ny, Nsim + 1);
    e = NaN(Ny, Nsim + 1);

    xhat(:,1) = pars.xhat0; % Poor initial guess
    L = cell(Nsim + 1, 1);
    P = cell(Nsim + 1, 1);
    P{1} = pars.P;

    % Loop for EKF State Estimation
    for t = 1:(Nsim + 1)
        % Measurement.
        yhat(:,t) = full(h(xhat(:,t)));
        e(:,t) = y(:,t) - yhat(:,t);

        % Correction
        C = full(Cfunc(xhat(:,t)));
        L{t} = (P{t}*C')/(C*P{t}*C' + Rv); % EKF gain
        P{t} = P{t} - L{t}*C*P{t};

        xhat(:,t) = xhat(:,t) + L{t}*e(:,t);

        % Implementing clipping
        if clipping
            xhat(:,t) = max(xhat(:,t), 0);
        end

        % Advance forecast.
        if t <= Nsim
            w = zeros(Nw, 1);

            A = full(Afunc(xhat(:,t), w));
            G = full(Gfunc(xhat(:,t), w));

            xhat(:,t + 1) = full(f(xhat(:,t), w));
            P{t + 1} = A*P{t}*A' + G*Qw*G';
        end
    end

    % Make a plot.
    pars.reactorplot(x, xhat, y, yhat, pars.yclean, pars.Delta);
    if clipping
        set(gca(), 'yscale', 'log', 'ylim', [1e-4, 1e3]);
    end

    % Form data table.
    data = [pars.t', x', xhat', y', yhat'];