function [x_sigma, w_sigma, v_sigma] = get_sigma(Pz, xhat, pars)
    % [x_sigma, w_sigma, v_sigma] = get_sigma(Pz, pars)
    %
    % Returns sigma points obtained from scaled sqrtm(Pz).
    narginchk(3, 3);

    % Make sure covariance is positive-definite.
    [Pzchol, okay] = chol(Pz);
    if okay ~= 0
        warning('Pz is not positive definite!');
    end

    % Select sigma points.
    sqrtPz = sqrtm((pars.Nz + pars.lambda)*Pz);
    zhat = [xhat; zeros(pars.Nw + pars.Nv, 1)];
    z_sigma = bsxfun(@plus, [zeros(pars.Nz, 1), sqrtPz, -sqrtPz], zhat);

    % Split into pieces.
    x_sigma = z_sigma(1:pars.Nx,:);
    w_sigma = z_sigma(pars.Nx + (1:pars.Nw),:);
    v_sigma = z_sigma(pars.Nx + pars.Nw + (1:pars.Nv),:);