% Example of nonconvex optmization using cooperative optimization.
nxy = 221;
xvec = linspace(-0.5, 5, nxy);
yvec = linspace(-0.5, 5, nxy);
ncont = 15;


[xgrid, ygrid] = meshgrid(xvec, yvec);
Vgrid = V(xgrid, ygrid);

Vfunc = @(u) V(u(1), u(2));

% Run distributed optimization for different initial conditions.
ulb = 0.1*[1; 1];
uub = 4*[1; 1];
U = [uub(1), ulb(1), ulb(1), uub(1), uub(1);
     uub(2), uub(2), ulb(1), ulb(1), uub(2)];

Nu = 2;
u0s = [0.5, 0.5; 3.9, 3.6; 2.99, 3]';
Nu0 = size(u0s, 2);
uiter = cell(Nu0, 1);
for i = 1:Nu0
    [~, uiter{i}] = distributedgradient(Vfunc, u0s(:,i), ulb, uub);
end

% Make a plot.
figure();
hold('on');
c = contour(xvec, yvec, Vgrid, ncont);
plot(U(1,:), U(2,:), '-k');
markers = 'os^<>v';
for i = 1:Nu0
    plot(uiter{i}(1,:), uiter{i}(2,:), ['-', markers(i), 'k']);
end
