function [x, u] = warmstart(sys, x, u, model)
    % Fills in a warm start using the terminal control law.
    Nt = size(u, 2);
    x = [x, NaN(length(x), Nt)];
    for t = 1:Nt
        if any(isnan(u(:,t)))
            u(:,t) = min(max(sys.K*x(:,t), sys.ulb), sys.uub);
        end
        switch model
        case 'nonlinear'
            x(:,t + 1) = sys.f(x(:,t), u(:,t));
        case 'linear'
            x(:,t + 1) = sys.A*x(:,t) + sys.B*u(:,t);
        otherwise
            error('Invalid choice for model!');
        end
    end