function errorplot2d(x, T, location)
% errorplot2d(x, T, [location])
%
% Makes a plot of integration error over the given time interval T.
%
% x must be a struct with fields to plot. All step sizes are assumed to be
% uniform. If x contains an 'exact' field, it is handled specially.
narginchk(2, 3);
if nargin() < 3
    location = {};
else
    location = {'location', location};
end

% Create figure and axes.
figure();

ax1 = subplot(2, 2, 1);
hold(ax1, 'on');
ylabel(ax1, 'x_1', 'rotation', 0);
xlabel(ax1, 't');

ax2 = subplot(2, 2, 3);
hold(ax2, 'on');
ylabel(ax2, 'x_2', 'rotation', 0);
xlabel(ax2, 't');

axp = subplot(2, 2, [2, 4]);
hold(axp, 'on');
ylabel(axp, 'x_2', 'rotation', 0);
xlabel(axp, 'x_1');

% Plot exact solution.
if isfield(x, 'exact')
    t = linspace(0, T, size(x.exact, 2));
    plot(ax1, t, x.exact(1,:), '-k');
    plot(ax2, t, x.exact(2,:), '-k');
    plot(axp, x.exact(1,:), x.exact(2,:), '-k');
    exact = {'exact'};
else
    exact = {};
end

% Plot other solutions.
fields = setdiff(fieldnames(x), {'exact'});
markers = {'o', 's', 'd', '^', 'v', '>', '<', 'p', 'h'};
markers = markers(mod(0:(length(fields) - 1), length(markers)) + 1);
colors = hsv(length(fields));
for i = 1:length(fields)
    f = fields{i};
    t = linspace(0, T, size(x.(f), 2));
    style = {markers{i}, 'color', colors(i,:)};
    plot(ax1, t, x.(f)(1,:), style{:});
    plot(ax2, t, x.(f)(2,:), style{:});
    plot(axp, x.(f)(1,:), x.(f)(2,:), style{:});
end

% Add legend.
legend(axp, exact{:}, fields{:}, location{:});

end%function
