% Example explicit numerical integration of a linear ODE.

% Butcher tableaus (only a and b) for the three methods.
tabs = struct();
tabs.euler = struct('a', 0, 'b', 1);
tabs.heun = struct('a', [0, 0; 1, 0], 'b', [0.5, 0.5]);
tabs.rk4 = struct('a', [0, 0, 0, 0; 0.5, 0, 0, 0; 0, 0.5, 0, 0; 0, 0, 1, 0], ...
                  'b', [1/6, 1/3, 1/3, 1/6]);
tabnames = fieldnames(tabs);

% Run each method.
A = [0, 1; -1, 0];
f = @(x) A*x;
x0 = [1; 0];
T = 2*pi();
Meval = 32;
x = struct();
for n = 1:length(tabnames)
    n = tabnames{n};
    Nstep = ceil(Meval/length(tabs.(n).b)); % Number of integration steps.
    h = T/Nstep;
    x.(n) = zeros(2, Nstep + 1);
    x.(n)(:,1) = x0;
    for i = 1:Nstep
        x.(n)(:,i + 1) = butcherintegration(x.(n)(:,i), f, h, ...
                                            tabs.(n).a, tabs.(n).b);
    end
end

% Add exact solution.
theta = linspace(0, 2*pi(), 257);
x.exact = [cos(theta); -sin(theta)];
tabnames = [{'exact'}; tabnames];

% Make a plot.
errorplot2d(x, T, 'SouthEast');

