% Accuracy of numerical integration of a stiff ODE.

% Butcher tableaus (only a and b) for the three methods.
tabs = struct();
tabs.ieu = struct('a', 1, 'b', 1);
tabs.mid = struct('a', 0.5, 'b', 1);
tabs.gl4 = struct('a', [1/4, 1/4 - sqrt(3)/6; 1/4 + sqrt(3)/6, 1/4], ...
                  'b', [1/2, 1/2]);
tabnames = fieldnames(tabs);

% Run each method for varying number of iterations.
x0 = [1; 0];
f = @stiffode;

% Choose number of function evaluations. Note that the figure in the text goes
% up to M = 10^7, which takes about an hour to run.
Ms = [1, 10, 100, 1000, 10000];

err = struct();
converged = struct();
for n = 1:length(tabnames)
    n = tabnames{n};
    err.(n) = NaN(size(Ms));
    converged.(n) = true(size(Ms));
    fprintf('Running %s.\n', n);
    for i = 1:length(Ms)
        M = Ms(i);
        fprintf('  M = %d\n', M);
        a = tabs.(n).a;
        b = tabs.(n).b;
        N = ceil(M/length(b)); % Number of integration steps.
        h = 2*pi()/N;
        x = x0;
        for t = 1:N
            [x, okay] = butcherintegration(x, f, h, a, b, 100);
            converged.(n)(i) = converged.(n)(i) && okay;
        end
        err.(n)(i) = norm(x - x0);
    end
end

% Make a plot.
figure();
hold('on');
colors = {'r', 'g', 'b'};
for n = 1:length(tabnames)
    c = colors{n};
    n = tabnames{n};
    loglog(Ms, err.(n), ['-o', c], 'displayname', n);
end
xlabel('Collocation points M');
ylabel('Error');
legend('location', 'northwest');

