% Example numerical integration of a stiff ODE.

% Butcher tableaus (only a and b) for the three methods.
tabs = struct();
tabs.ieu = struct('a', 1, 'b', 1);
tabs.mid = struct('a', 0.5, 'b', 1);
tabs.gl4 = struct('a', [1/4, 1/4 - sqrt(3)/6; 1/4 + sqrt(3)/6, 1/4], ...
                  'b', [1/2, 1/2]);
tabnames = fieldnames(tabs);

% Run each method.
f = @stiffode;
x0 = [1; 0];
Neval = 10;
T = 2*pi();
x = struct();
for n = 1:length(tabnames)
    n = tabnames{n};
    Nstep = Neval/length(tabs.(n).b); % Use Neval function evaluations.
    h = T/Nstep;
    x.(n) = zeros(2, Nstep + 1);
    x.(n)(:,1) = x0;
    for i = 1:Nstep
        x.(n)(:,i + 1) = butcherintegration(x.(n)(:,i), f, h, ...
                                            tabs.(n).a, tabs.(n).b);
    end
end

% Add exact solution.
theta = linspace(0, 2*pi(), 257);
x.exact = [cos(theta); -sin(theta)];
tabnames = [{'exact'}; tabnames];

% Make a plot.
errorplot2d(x, T, 'SouthEast');

