function mpc_book_setup(ifmissing)
% mpc_book_setup()
% mpc_book_setup('warn')
% mpc_book_setup('error')
% mpc_book_setup('ignore')
%
% Adds the necessary directories to Octave/Matlab's path and checks for
% Casadi and MPCTools. Argument specifies what to do if Casadi/MPCTools are not
% found.
narginchk(0, 1);
if nargin() < 1
    ifmissing = 'warn';
end

% Add the booklib path.
thisdir = fileparts(mfilename('fullpath'));
addpath([thisdir, '/booklib']);

% Load the control package.
if isOctave()
  pkg('load', 'control');
  pkg('load', 'optim');
end%if

% Look for MPCTools and Casadi.
directories = {'casadi', 'mpctools'};
for i = 1:length(directories)
    directory = [thisdir, '/', directories{i}];
    if exist(directory, 'dir')
        addpath(directory);
    end
end

% Issue errors, warnings, etc.
switch ifmissing
case 'error'
    missing = @error;
case 'warn'
    missing = @warning;
case 'ignore'
    missing = @(varargin) [];
otherwise
    error('Invalid argument!');
end

if isempty(which('casadiMEX'))
    missing(['Casadi not found. Download from files.casadi.org and unzip ' ...
             'to a folder called ''casadi''.']);
end

if isempty(which('import_mpctools'))
    missing(['MPCTools not found. Download from bitbucket.org/' ...
             'rawlings-group/octave-mpctools and unzip to a folder called ' ...
             '''mpctools''.']);
end

end%function

